<?php
/**
 * @package The_SEO_Framework\Classes\Sitemap\Ping
 * @subpackage The_SEO_Framework\Sitemap
 */

namespace The_SEO_Framework\Sitemap;

\defined( 'THE_SEO_FRAMEWORK_PRESENT' ) or die;

use function \The_SEO_Framework\memo;

use \The_SEO_Framework\Data;

/**
 * The SEO Framework plugin
 * Copyright (C) 2023 - 2024 Sybre Waaijer, CyberWire B.V. (https://cyberwire.nl/)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Holds search engine sitemap pinging functionality.
 *
 * @since 4.0.0
 * @since 5.0.0 Moved from `\The_SEO_Framework\Bridges`.
 * @access protected
 *         Use tsf()->sitemap()->ping() instead.
 */
class Ping {

	/**
	 * Prepares a cronjob-based ping within 30 seconds of calling this.
	 *
	 * @since 4.0.0
	 * @since 4.1.0 Now returns whether the cron engagement was successful.
	 * @since 4.1.2 Now registers before and after cron hooks. They should run subsequentially when successful.
	 * @see static::engage_pinging_retry_cron()
	 *
	 * @return bool True on success, false on failure.
	 */
	public static function engage_pinging_cron() {

		$when = time() + 28;

		// Because WordPress sorts the actions, we can't be sure if they're scrambled. Therefore: skew timing.
		// Note that when WP_CRON_LOCK_TIMEOUT expires, the subsequent actions will run, regardless if previous was successful.
		return \wp_schedule_single_event( ++$when, 'tsf_sitemap_cron_hook_before' )
			&& \wp_schedule_single_event( ++$when, 'tsf_sitemap_cron_hook' )
			&& \wp_schedule_single_event( ++$when, 'tsf_sitemap_cron_hook_after' );
	}

	/**
	 * Retries a cronjob-based ping, via another hook.
	 *
	 * @since 4.1.2
	 * @uses \WP_CRON_LOCK_TIMEOUT, default 60 (seconds).
	 *
	 * @param array $args Optional. Array containing each separate argument to pass to the hook's callback function.
	 * @return bool True on success, false on failure.
	 */
	public static function engage_pinging_retry_cron( $args = [] ) {

		$when = (int) ( time() + min( \WP_CRON_LOCK_TIMEOUT, 60 ) + 1 );

		return \wp_schedule_single_event( $when, 'tsf_sitemap_cron_hook_retry', [ $args ] );
	}

	/**
	 * Retries pinging the search engines for the base sitemap only.
	 *
	 * Devs: If you need to retry pinging another sitemap, please engage a custom scheduler,
	 *       first call `engage_pinging_retry_cron( [ 'id' => 'my_sitemap_id' ] )`, then hook
	 *       into action `tsf_sitemap_cron_hook_retry` and test `$args['id']` like below.
	 *       Alternatively, hitch with `the_seo_framework_ping_search_engines`.
	 *
	 * @since 4.1.2
	 *
	 * @param array $args Array from ping hook.
	 */
	public static function retry_ping_search_engines( $args = [] ) {

		if ( empty( $args['id'] ) || 'base' !== $args['id'] ) return;

		static::ping_search_engines();
	}

	/**
	 * Pings search engines the base sitemap.
	 *
	 * @since 2.2.9
	 * @since 2.8.0 Only worked when the blog was not public...
	 * @since 3.1.0 Now allows one ping per language.
	 * @since 3.2.3 1. Now works as intended again.
	 *              2. Removed Easter egg.
	 * @since 4.0.0 Moved to \The_SEO_Framework\Bridges\Ping
	 * @since 4.0.2 Added action.
	 * @since 4.1.1 Added another action.
	 *
	 * @return void Early if blog is not public.
	 */
	public static function ping_search_engines() {

		if ( Data\Plugin::get_option( 'site_noindex' ) || ! Data\Blog::is_public() ) return;

		// Check for sitemap lock. If TSF's default sitemap isn't used, this should return false.
		if ( Lock::is_sitemap_locked( 'base' ) ) {
			static::engage_pinging_retry_cron( [ 'id' => 'base' ] );
			return;
		}
		$transient = Cache::build_sitemap_cache_key( 'tsf_throttle_ping' );

		// Uses legacy get_transient to bypass TSF's transient filters and prevent ping spam.
		if ( false === \get_transient( $transient ) ) {
			/**
			 * @since 4.1.1
			 * @param string $class The current class name.
			 */
			\do_action( 'the_seo_framework_before_ping_search_engines', static::class );

			if ( Data\Plugin::get_option( 'ping_google' ) )
				static::ping_google();

			if ( Data\Plugin::get_option( 'ping_bing' ) )
				static::ping_bing();

			/**
			 * @since 4.0.2
			 * @param string $class The current class name.
			 */
			\do_action( 'the_seo_framework_ping_search_engines', static::class );

			/**
			 * @since 2.5.1
			 * @param int $expiration The minimum time between two pings.
			 */
			$expiration = (int) \apply_filters( 'the_seo_framework_sitemap_throttle_s', \HOUR_IN_SECONDS );

			// Uses legacy set_transient to bypass TSF's transient filters and prevent ping spam.
			\set_transient( $transient, 1, $expiration );
		}
	}

	/**
	 * Pings the main sitemap location to Google.
	 *
	 * @since 2.2.9
	 * @since 3.1.0 Updated ping URL. Old one still worked, too.
	 * @since 4.0.0 Moved to \The_SEO_Framework\Bridges\Ping
	 * @since 4.0.3 Google now redirects to HTTPS. Updated URL scheme to accommodate.
	 * @since 4.1.2 Now fetches WP Sitemaps' index URL when it's enabled.
	 * @link https://developers.google.com/search/docs/advanced/crawling/ask-google-to-recrawl
	 */
	public static function ping_google() {

		$url = static::get_ping_url();

		if ( empty( $url ) ) return;

		\wp_safe_remote_get(
			'https://www.google.com/ping?sitemap=' . rawurlencode( $url ),
			[ 'timeout' => 3 ],
		);
	}

	/**
	 * Pings the main sitemap location to Bing.
	 *
	 * @since 2.2.9
	 * @since 3.2.3 Updated ping URL. Old one still worked, too.
	 * @since 4.0.0 Moved to \The_SEO_Framework\Bridges\Ping
	 * @since 4.0.3 Bing now redirects to HTTPS. Updated URL scheme to accommodate.
	 * @since 4.1.2 Now fetches WP Sitemaps' index URL when it's enabled.
	 * @link https://www.bing.com/webmasters/help/Sitemaps-3b5cf6ed
	 */
	public static function ping_bing() {

		$url = static::get_ping_url();

		if ( empty( $url ) ) return;

		\wp_safe_remote_get(
			'https://www.bing.com/ping?sitemap=' . rawurlencode( $url ),
			[ 'timeout' => 3 ],
		);
	}

	/**
	 * Return the base sitemap's ping URL.
	 * Memoizes the return value.
	 *
	 * @since 4.2.0
	 * @since 5.0.0 Added first `$sitemap_id` parameter.
	 *
	 * @param string $sitemap_id The sitemap ID. Only works when the Optimized Sitemaps are enabled.
	 * @return string The ping URL. Empty string on failure.
	 */
	public static function get_ping_url( $sitemap_id = 'base' ) {
		return memo( null, $sitemap_id ) ?? memo(
			(
				Utils::use_core_sitemaps()
					? \get_sitemap_url( 'index' )
					: Registry::get_expected_sitemap_endpoint_url( $sitemap_id )
			)
			?: '',
			$sitemap_id,
		);
	}
}
